// T3DFrameView.cpp
/*
* $Log: T3DFrameView.cpp,v $
* Revision 1.1  1997/04/05 15:58:52  elvis
* Initial revision
*
*/

#include "T3DFrameView.h"
#include "colors.h"

T3DFrameView::T3DFrameView(BRect inFrame, const char *inName, ulong inResizingMode, long depth, ulong sides)
: TSwissArmyView(inFrame, inName, inResizingMode, B_WILL_DRAW)
{
	mSides = sides;
	mDepth = depth;
	
	light = white;
	dark = black;
	mid = dkGray;
	
	mUseMid = true;
	mInverted = false;
	
	mLeftLabelLead = 8;
	
	mDrawLabel = true;
	mLabelCentered = false;
}

void
T3DFrameView::Draw(BRect inRect)
{
	long i;
	BRect theRect = Bounds();
	long	theDepth;
	if ( mDrawLabel )
	{
		theRect.top += 5;
		
		SetHighColor(black);
		DrawString(Name(), BPoint( mLeftLabelLead + 4,9));
	}
	
	if ( mHighlight > 0 )
	{
		theDepth = mDepth + 1;
	}
	else
	{
		// Erase the Highlight
		rgb_color theViewColor = ViewColor();
		BeginLineArray(5);
		AddLine( BPoint( theRect.left, theRect.top), BPoint (theRect.left + mLeftLabelLead,theRect.top), theViewColor);
		AddLine( BPoint( theRect.left + mLabelOffset, theRect.top), BPoint (theRect.right,theRect.top), theViewColor);
		AddLine( BPoint( theRect.left, theRect.top), BPoint (theRect.left,theRect.bottom), theViewColor);
		AddLine( BPoint( theRect.right, theRect.top), BPoint (theRect.right,theRect.bottom), theViewColor);
		AddLine( BPoint( theRect.left, theRect.bottom), BPoint (theRect.right,theRect.bottom), theViewColor);
		EndLineArray();
		theRect.InsetBy(1,1);
		theDepth = mDepth;
	}

	if ( mUseMid )
	{
		if ( mInverted == false )
		{
			if ( mSides & T_LEFT_SIDE )
			{
				BeginLineArray(theDepth);
				for ( i = 0; i< theDepth; i++)
				{
					AddLine( BPoint( theRect.left + i, theRect.top+i), BPoint ( theRect.left+i,theRect.bottom-i), light);
				}
				EndLineArray();
					
			}
					
			if ( mSides & T_TOP_SIDE )
			{
				BeginLineArray(theDepth * 2);
				for ( i = 0; i< theDepth; i++)
				{
					AddLine( BPoint( theRect.left + i, theRect.top+i), BPoint ( theRect.left + mLeftLabelLead,theRect.top+i), light);
					AddLine( BPoint( theRect.left + mLabelOffset, theRect.top+i), BPoint ( theRect.right-i,theRect.top+i), light);
				}
				EndLineArray();
					
			}
			
			if ( mSides & T_RIGHT_SIDE )
			{
				BeginLineArray(theDepth+1);
				for ( i = 0; i< theDepth; i++)
				{
					AddLine( BPoint( theRect.right - i, theRect.top+i+1), BPoint ( theRect.right-i,theRect.bottom-i), dark);
				}
				AddLine(BPoint( theRect.right - theDepth , theRect.top+theDepth), BPoint ( theRect.right-theDepth ,theRect.bottom-theDepth), mid);
				EndLineArray();
					
			}
		
			if ( mSides & T_BOTTOM_SIDE )
			{
				BeginLineArray(theDepth+1);
				for ( i = 0; i< theDepth; i++)
				{
					AddLine( BPoint( theRect.left + i +1, theRect.bottom-i), BPoint ( theRect.right-i-1,theRect.bottom-i), dark);
				}
				AddLine(BPoint( theRect.left + theDepth , theRect.bottom-theDepth), BPoint ( theRect.right-theDepth,theRect.bottom-theDepth), mid);
				EndLineArray();
					
			}
		}
		else
		{	
			if ( mSides & T_LEFT_SIDE )
			{
				BeginLineArray(theDepth+1);
				for ( i = 0; i< theDepth; i++)
				{
					AddLine( BPoint( theRect.left + i, theRect.top+i), BPoint ( theRect.left+i,theRect.bottom-i), dark);
				}
				AddLine( BPoint( theRect.left + theDepth, theRect.top+theDepth), BPoint ( theRect.left+theDepth,theRect.bottom-theDepth), mid);
				EndLineArray();
					
			}
					
			if ( mSides & T_TOP_SIDE )
			{
				BeginLineArray(theDepth*2 + 2);
				for ( i = 0; i< theDepth; i++)
				{
					AddLine( BPoint( theRect.left + i, theRect.top+i), BPoint ( theRect.left + mLeftLabelLead,theRect.top+i), dark);
					AddLine( BPoint( theRect.left + mLabelOffset, theRect.top+i), BPoint ( theRect.right-i,theRect.top+i), dark);
				}
				AddLine( BPoint( theRect.left +theDepth, theRect.top+theDepth), BPoint ( theRect.left + mLeftLabelLead,theRect.top+theDepth), mid);
				AddLine( BPoint( theRect.left +mLabelOffset, theRect.top+theDepth), BPoint ( theRect.right-theDepth,theRect.top+theDepth), mid);
				EndLineArray();
					
			}
			
			if ( mSides & T_RIGHT_SIDE )
			{
				BeginLineArray(theDepth);
				for ( i = 0; i< theDepth; i++)
				{
					AddLine( BPoint( theRect.right - i, theRect.top+i+1), BPoint ( theRect.right-i,theRect.bottom-i), light);
				}
				EndLineArray();
					
			}
		
			if ( mSides & T_BOTTOM_SIDE )
			{
				BeginLineArray(theDepth);
				for ( i = 0; i< theDepth; i++)
				{
					AddLine( BPoint( theRect.left + i +1, theRect.bottom-i), BPoint ( theRect.right-i-1,theRect.bottom-i), light);
				}
				EndLineArray();
					
			}
		}
	}
	else // mUseMid
	{
		rgb_color l,d;
		
		if ( mInverted == false )
		{
			l = light;
			d = dark;
		}
		else
		{
			l = dark;
			d = light;
		}
		
		if ( mSides & T_LEFT_SIDE )
		{
			BeginLineArray(theDepth);
			for ( i = 0; i< theDepth; i++)
			{
				AddLine( BPoint( theRect.left + i, theRect.top+i), BPoint ( theRect.left+i,theRect.bottom-i), l);
			}
			EndLineArray();
				
		}
				
		if ( mSides & T_TOP_SIDE )
		{
			BeginLineArray(theDepth*2);
			for ( i = 0; i< theDepth; i++)
			{
				AddLine( BPoint( theRect.left + i, theRect.top+i), BPoint ( theRect.left+mLeftLabelLead,theRect.top+i), l);
				AddLine( BPoint( theRect.left + mLabelOffset, theRect.top+i), BPoint ( theRect.right-i,theRect.top+i), l);
			}
			EndLineArray();
				
		}
		
		if ( mSides & T_RIGHT_SIDE )
		{
			BeginLineArray(theDepth);
			for ( i = 0; i< theDepth; i++)
			{
				AddLine( BPoint( theRect.right - i, theRect.top+i+1), BPoint ( theRect.right-i,theRect.bottom-i), d);
			}
			EndLineArray();
				
		}
	
		if ( mSides & T_BOTTOM_SIDE )
		{
			BeginLineArray(theDepth);
			for ( i = 0; i< theDepth; i++)
			{
				AddLine( BPoint( theRect.left + i +1, theRect.bottom-i), BPoint ( theRect.right-i-1,theRect.bottom-i), d);
			}
			EndLineArray();
				
		}
	}
	
}

void
T3DFrameView::CalculateTextPosition()
{
	const char *theName = Name();
	if ( !mDrawLabel  && *theName == '\0' )
	{
		mLabelWidth = 0;
		mLabelOffset = 0;
	}	
	else
	{
		mLabelWidth = StringWidth(theName);	
		if ( mLabelCentered )
		{
			mLeftLabelLead =Frame().Width() / 2 - mLabelWidth / 2 - 2;
			
			mLabelOffset = mLeftLabelLead + 4 + mLabelWidth; 
		}
		else
		{
			mLabelOffset = mLeftLabelLead + 5 + mLabelWidth; 
		}
	}
}

void
T3DFrameView::AttachedToWindow()
{
	CalculateTextPosition();
}
	