// T3DFrameView.h 
// Trevor Walker

// This class draws a 3D framed view.  It can be labeled.  If it
// is labeled, the top is set down approiately so the text is not
// clipped.  

// Dependencies:
//	TSwissArmyView (Direct)
//	TCaptionWindow (Indirect)

// Options include:
//	Inverted or not
//  Secondary shadow line on dark side ( see SetMid and SetUseMid )
//  Optional Label 
//  Optional Depth ( upto 10 )
//  Option to draw only certain sides

// Note: Note all options are applicapable with all others.  Notably,
// the label option does not work with depths > 1 or fonts > 12 points.

// Make sure you lower child views appropriately when placing them.
// Make sure that if you change the font, call CalculateTextPosition to 
//	reset things.
//

// Use of this code is permitted in all shareware as long as credit
// is given.  For use in commercial software please contact me at
// twalker@cc.edu.
 
#pragma once

#include "TSwissArmyView.h"

enum { T_LEFT_SIDE = 1, T_TOP_SIDE = 2, T_RIGHT_SIDE = 4, T_BOTTOM_SIDE = 8,T_ALL_SIDES=15};

class T3DFrameView : public TSwissArmyView
{
public:
	T3DFrameView(BRect inFrame, const char *inName, ulong inResizingMode,long depth, ulong sides);
	
	virtual	void	Draw(BRect inRect);
	virtual void	AttachedToWindow();
	
			void	SetDepth(long inDepth) {mDepth = inDepth;}
			void	SetUseMid(bool m) { mUseMid = m;}
			void	SetLight(rgb_color c) { light = c;}
			void	SetDark(rgb_color c) { dark = c;}
			void	SetMid(rgb_color c) { mid = c;}
			void	SetInverted(bool i) { mInverted =i; }
			void	SetLabelCentered(bool c) { mLabelCentered=c; }
			void	SetLeftLabelLead(long l) { mLeftLabelLead = l; }
			
			void	CalculateTextPosition();
private:
	ulong	mSides;
	long	mDepth;
	bool	mUseMid;  // Use Secondary Middle Gray 
	bool	mInverted;
	bool	mDrawLabel;
	bool	mLabelCentered;
	long	mLabelWidth,mLabelOffset,mLeftLabelLead;
	rgb_color	light,mid,dark;
};