// TCaptionWindow.cpp
/*
* $Log: TCaptionWindow.cpp,v $
* Revision 1.1  1997/04/05 15:58:52  elvis
* Initial revision
*
*/

#include "TCaptionWindow.h"

#include "colors.h"

double	TCaptionWindow::sLastShown = 0;
double	TCaptionWindow::sDisplayThreshold = 1000000;
double	TCaptionWindow::sRedisplayThreshold = 330000;
double  TCaptionWindow::sHideThreshold = 7000000;
double  TCaptionWindow::sCloseThreshold = 10000000;

pattern TCaptionWindow::sPattern = { 0x22, 0x88, 0x22, 0x88, 0x22, 0x88, 0x22, 0x88 };
rgb_color TCaptionWindow::sColor = { 238,238,100 };

TCaptionWindow*	gCaptionWindow = NULL;

TCaptionWindow::TCaptionWindow()
	: BWindow ( BRect(0,0,5,5), "_caption_window", B_BORDERED_WINDOW, 
				B_NOT_MOVABLE|B_NOT_RESIZABLE|B_NOT_CLOSABLE|
				B_NOT_ZOOMABLE|B_NOT_MINIMIZABLE|B_WILL_FLOAT )				
{
	if ( gCaptionWindow == NULL )
		gCaptionWindow = this;
		
	Lock();

	SetPulseRate(sDisplayThreshold);

	mView = new TCaptionView("");
	AddChild(mView);
	
	mTimeEntered = 0;
	mTimeExited = 0;
	mTimeShown = 0;
	mTimeHidden = 0;
	mAlreadyShown = false;
	mCaller = NULL;
	
	mPulser = spawn_thread(Pulser, "_caption_pulse", B_NORMAL_PRIORITY, this);
	if ( mPulser > 0 )
		resume_thread(mPulser);
	
	Unlock();
}

TCaptionWindow::~TCaptionWindow()
{
	if ( gCaptionWindow == this )
		gCaptionWindow = NULL;
		
	kill_thread(mPulser);
}

void
TCaptionWindow::ViewEntered(BView *inView, BRect inRect, const char *inCaption)
{
	// This is called by the view every time the mouse enters the view.  The only way that a
	// view can be entered is through a B_VIEW_ENTERED mouse moved event.  This function should
	// be called every time a B_VIEW_ENTERED is encountered.
	
	// Make sure inRect is in global coordinates.
	
	if ( inView != mCaller )
	{
		// This is a new view, so set things up
		mCaller = inView;
		mTimeEntered = system_time();
		mTimeExited = 0;
		mCallerRect = inRect;
				
		SetCaption(inCaption);

		BPoint 	thePoint;
		ulong	theButton;
		
		mView->FakeGetMouse(&thePoint, &theButton);
		
		if ( mTimeEntered - mTimeHidden < sRedisplayThreshold && mCallerRect.Contains(thePoint))
			DoShow();
	}
}

void
TCaptionWindow::ViewExited(BView *inView, BPoint inPoint, bool inForce)
{
	// This should be called by the view every time a B_VIEW_EXITED mouse moved event occurs.
	// It will check to see if the view was actually exit (i.e. the mouse moved outside the
	// bounds rect of the view) or it the event was generated by moving into the help window,
	// which isn't really leaving the view.
	
	// This should also be called by when mouse down occur to hide the help.
	
	// Make sure inPoint is in global coordinates.
	
	if ( inForce || ( mCaller == inView && ! mCallerRect.Contains(inPoint) ) )
	{
		// The mouse is not actually outside of the view
		DoExited();
	}
}

void
TCaptionWindow::SetCaption(BView* inView, const char* inCaption)
{		
	if ( mCaller == inView )
	{
		SetCaption(inCaption);
	}

}

void
TCaptionWindow::SetCaption(const char* inCaption)
{
	Lock();
	mView->SetCaption(inCaption);

	DoRelocate();
	
	Unlock();
}

void
TCaptionWindow::DoRelocate()
{
	BRect theRect = mView->Size();
	ResizeTo(theRect.Width(),theRect.Height());

	BPoint thePoint;
	ulong	theButton;
	
	mView->FakeGetMouse(&thePoint, &theButton);
	
	screen_info theInfo;
	
	get_screen_info(&theInfo);
	
	if (thePoint.x + theRect.Width() > theInfo.frame.Width() ) 
		thePoint.x = theInfo.frame.Width() - theRect.Width() - 5;
	
	thePoint.y += 20;
	if ( thePoint.y + theRect.Height() > theInfo.frame.Height() ) thePoint.y -= 40;
	
	MoveTo(thePoint);
}

void
TCaptionWindow::DoPulse()
{
	double theTime = system_time();
	
	if ( mTimeExited && theTime - mTimeExited > sCloseThreshold )
	{
		delete this;
	}
	else
	if ( mTimeShown )
	{
		if ( theTime - mTimeShown > sHideThreshold )
		{
			DoHide();
		}
	}
	else // View is hidden
	if ( mCaller && ! mAlreadyShown)
	{
		if ( theTime - mTimeEntered > sDisplayThreshold )
		{
			DoShow();
		}
	}
} 

/*void
TCaptionWindow::WindowActivated(bool inActive)
{
	if ( inActive == false && mTimeShown)
		DoHide();
}*/

void
TCaptionWindow::MouseMoved(BPoint inPoint)
{
	// Checks to make sure the mouse doesn't leave a view while it is in the help window

	if ( mCaller && ! mCallerRect.Contains(inPoint) )
	{
		DoExited();
	}
	else
	{
		DoRelocate();
	}
}

void
TCaptionWindow::DoShow()
{
	Lock();
	
	DoRelocate();
	
	Unlock();
	
	mTimeShown = system_time();
	mTimeHidden = 0;
	
	mAlreadyShown = true;
	
	Show();	

}

void
TCaptionWindow::DoHide()
{
	mTimeShown = 0;
	mTimeHidden = system_time();
	Hide();
}
			
void
TCaptionWindow::DoExited()
{
	// Called when the mouse actually leaves the view, i.e. is outside the bounds rect.
	
	if ( mTimeShown )
		DoHide();
		
	mCaller = 0;
	mTimeEntered = 0;
	mTimeExited = system_time();
	
	mAlreadyShown = false;
}

long 
Pulser(void *inPtr)
{
	double thePace;
	
	if ( gCaptionWindow == NULL )
		exit(0);
	
	thePace = gCaptionWindow->DisplayThreshold();
	
	while ( 1 )
	{
		snooze ( thePace );
		
		((TCaptionWindow*)inPtr) -> DoPulse();
	}
}

TCaptionView::TCaptionView(const char *inCaption)
	: BView ( BRect(0,0,5,5) , "CaptionView" , B_FOLLOW_ALL, B_WILL_DRAW)
{
	strncpy(mCaption, inCaption, 50);
}	
	
void
TCaptionView::Draw(BRect inRect)
{
	SetHighColor( ((TCaptionWindow*) Window())->Color() );
	
	FillRect(Bounds(), ((TCaptionWindow*) Window())->Pattern() );
	
	SetHighColor( black );
	DrawString(mCaption,BPoint(3,10));	
}

BRect
TCaptionView::Size()
{
	long theWidth = StringWidth( mCaption );
	
	return BRect ( 0,0,theWidth+6,14);
}

void
TCaptionView::SetCaption(const char *inCaption)
{
	strncpy(mCaption, inCaption, 50);
	Invalidate();
}

void
TCaptionView::FakeGetMouse(BPoint *inPoint, ulong *inButtons)
{
	Window()->Lock();
	GetMouse(inPoint,inButtons);
	ConvertToScreen(inPoint);
	Window()->Unlock();
}

void
TCaptionView::MouseMoved(BPoint inPoint, ulong inState, BMessage *inMsg)
{

	if ( Window() )
		((TCaptionWindow*) Window())->MouseMoved(ConvertToScreen(inPoint));
}

	

