// TCaptionWindow.h

// Popup Help caption (as made popular by Microsoft).  Currently very buggy.

// Using:
//	Whenever the mouse enters the view, first check the gCaptionWindow
//	variable.  If it is zero, create a new TCaptionWindow.  Otherwise use
//	the one that already exists.  After it is created, call 
//	gCaptionWindow->ViewEntered.
//	Whenever the mouse leave the view, call gCaptionWindow->ViewExited.
// 	Example of MouseMoved()
//
//ViewClass::MouseMoved(BPoint inPoint, ulong inState, BMessage *inMsg)
//{
//	if ( inState == B_ENTERED_VIEW )
//	{
//		if ( inMsg == NULL )
//		{
//			Highlight(T_MOUSEOVER_HIGHLIGHT);
//			
//			// Check for caption display
//			if ( mCaptionEnabled )
//			{			
//				if ( gCaptionWindow == NULL )
//					new TCaptionWindow();
//				
//				gCaptionWindow->ViewEntered(this,ConvertToScreen(Bounds()),mCaption);	
//					
//			}
//		}	
//		
//	}
//	else
//	if ( inState == B_EXITED_VIEW   )
//	{
//		if (mHighlight != T_NO_HIGHLIGHT)
//			Highlight(T_NO_HIGHLIGHT);
//			
//		if ( gCaptionWindow )
//		{
//			gCaptionWindow->ViewExited(this, ConvertToScreen(inPoint));
//		}
//	}
//}

// The behavior of the caption is controlled by the following static variables:
//	sDisplayThreshold -- the amount of time in msec that the cursor must be in a
//							view before the caption is shown. Default 1 second.
//	sRedisplayThreshold -- the maximum time between hiding the caption and immediately
//							redisplaying it if a new view is entered. Default 1/3 second.
//	sHideThreshold -- the amount of time in msec before the caption is hidden after
//						after it has been displayed.  Change this to make the caption
//						stay on the screen longer.  Default 7 seconds.
//	sCloseThreshold -- the amount of time in msec after the last hiding of the caption
//						that gCaptionView will be destroyed.  Default 10 seconds.


// Problems with TCaptionView
//	--Shows up sometimes when the cursor is no longer in the view.
//	--Doesn't hide when mouse is clicked.  Window must be accept first click
//		in order for controls to work in the most desirable manner.
//	--Doesn't hide when the caption is no longer the formost window (if the cursor 
//		is still in the view).
//	--Caption Doesn't flaot (BeOS problem as of DR8).

// For further information, set comments below.

// Use of this code is permitted in all shareware as long as credit
// is given.  For use in commercial software please contact me at
// twalker@cc.edu.

#pragma once 


class TCaptionView;

class TCaptionWindow : public BWindow
{
public:
			TCaptionWindow();
	virtual ~TCaptionWindow();
	virtual void	MouseMoved(BPoint inPoint);
//	virtual void	WindowActivated(bool);

			void	SetCaption(BView *inView, const char* inCaption);
						// Set the caption.  If the help is currently shown
						// this will change the displayed caption.
			void	ViewEntered(BView *inView, BRect inRect, const char *inCaption);
						// Call this whenever the view is entered.  
						// inRect should be the views frame in screen
						// coordinates.  inCaption will be the caption displayed.
			void	ViewExited(BView *inView, BPoint inPoint, bool inForce = false);
						// Call whenever the view is exited  for any reason.  
						// This will handle the situation where the view was
						// exited because the mouse moved into a shown caption.
						// inPoint should be the current mouse position in global
						// coordinates.  
						// if inForce is set to true, the caption will be closed.

// Control of Variables			
	static	void	SetLastShown(double s) { sLastShown = s;}
	static 	void	SetDisplayThreshold(double t) { sDisplayThreshold = t; }
	static	void	SetRedisplayThreshold(double t) { sRedisplayThreshold = t; }
	static	void	SetPattern(pattern p) { sPattern = p; }
	static	void	SetColor(rgb_color c) { sColor = c; }

// Get info	
	static	double	LastShown() {return sLastShown; }
	static	double	DisplayThreshold() {return sDisplayThreshold; }
	static	double	RedisplayThreshold() { return sRedisplayThreshold; }
	static	pattern	Pattern() { return sPattern; }
	static	rgb_color	Color() { return sColor; }			
		
		
			void	DoPulse();  // Should be private.  Don't call this.

private:
			void	DoShow();
			void	DoHide();
			void	DoExited();
			void	DoRelocate();
			void	SetCaption(const char*);

TCaptionView *mView;
static	double	sLastShown;
static	double	sDisplayThreshold;
static	double	sRedisplayThreshold;
static	double	sHideThreshold;
static	double	sCloseThreshold;
static  pattern sPattern;
static	rgb_color sColor;

		BView	*mCaller;	// View that last called.
		BRect	mCallerRect;
		double	mTimeEntered;	// Time mCaller was first entered.
		double	mTimeExited;	// Time mCaller was last exited.
		double  mTimeShown;	// Time at which the Caption was shown.  if mTimeShown != 0, 
							// caption is visible.
		double	mTimeHidden;	// Time at which caption was hidden.
		bool	mAlreadyShown;
		long	mActivateFlag;
		
		thread_id mPulser;

};

class TCaptionView : public BView
{
public:
	TCaptionView(const char *inCaption);
	
	virtual	void	Draw(BRect inRect);
			BRect	Size();
			void	SetCaption(const char *inCaption);
			void	FakeGetMouse(BPoint*,ulong*);
	virtual void	MouseMoved(BPoint,ulong,BMessage*);
private:
	char	mCaption[50];
};	

long Pulser(void *inPtr);

extern	TCaptionWindow *gCaptionWindow;
