/*
* $Log: TGraphView.cpp,v $
* Revision 1.1  1997/04/05 15:58:52  elvis
* Initial revision
*
*/

#include "TGraphView.h"

#include "colors.h"

TGraphView::TGraphView(BRect frame, char *name, ulong mode)
	: BView(frame, name, mode, B_WILL_DRAW|B_FRAME_EVENTS|B_FULL_UPDATE_ON_RESIZE)
{

	mWidth = frame.IntegerWidth();
	mHeight = frame.IntegerHeight();

	mLineColor = red;
	mViewColor = mdGray;
	mGridColor= dkGray;
	
	// Setup the Bitmap and View for drawing
	frame.OffsetTo(0,0);
	mBitmap = new BBitmap(frame,B_COLOR_8_BIT,true);
	mView = new BView(frame, "", B_FOLLOW_ALL, NULL);
	
	mBitmap->Lock();
	mBitmap->AddChild(mView);
	mView->SetHighColor(mViewColor);
	mView->FillRect(frame);
	mBitmap->Unlock();

	// Setup the other variables
	mMax = 100;
	mStart = 0;
	mNumOfPoints = 0;
	mHScale = 5;
	mVScale = (float)mHeight / mMax;
	mDataWidth = mWidth / mHScale + 1;
	mMinMax = 100;
	mGridInterval = 25;
	mLineWidth = 1;
	
	
	mCurrentPosition = 0;
	mNextPosition = mCurrentPosition + mHScale;
	
	
	// Allocate the Data Array
	mPoints = new long[mDataWidth];
	
	for (long i=0;i< mDataWidth; i++)
	{
		mPoints[i] = 0;
	}
		
}

TGraphView::~TGraphView()
{
	mBitmap->Lock();
	delete mBitmap;
	delete mPoints;
}

void
TGraphView::Draw(BRect bounds)
{
	DrawBitmap(mBitmap, bounds, bounds);
}


void
TGraphView::AddPoint(long inPoint)
{
	long theNewPosition;
	long theNewMax = -1;
	
	if ( mNumOfPoints == mDataWidth )
	{	
		// Can't Add a new point, so rotate the points.
		theNewPosition = mStart;
		mStart = (mStart + 1) % mDataWidth;
	//	printf ("start: %d, position: %d\n",mStart,theNewPosition);

		if ( inPoint > mMax )
		{
			theNewMax = inPoint;
		}
		else
		if ( mPoints[theNewPosition] == mMax)
		{
			// Replace the data point with the new one
			mPoints[theNewPosition] = inPoint;
			theNewMax = mMinMax;
			for (long i=0;i<mNumOfPoints;i++)
			{
				if ( mPoints[i] > theNewMax )
				{
					theNewMax = mPoints[i];
				}
			}
		}	
	}
	else
	{
		theNewPosition = ( mStart + mNumOfPoints ) % mDataWidth;  
		mNumOfPoints++;
												
		if ( inPoint > mMax )
			theNewMax = inPoint;
			
	}
	
	mPoints[theNewPosition] = inPoint;
	
	if ( theNewMax != -1 )
	{
		// Max Changed.  Must redraw Everything
		mMax = theNewMax;
		mVScale = (float)mHeight / mMax;
		DrawEverything();
		Window()->Lock();
		Draw(Bounds());
		Window()->Unlock();	

	}
	else
	if ( mNumOfPoints == 1 )
	{
		DrawEverything();
		Window()->Lock();
		Draw(Bounds());
		Window()->Unlock();	
	}
	else
	{
		if ( mNumOfPoints > 1 ) // Don't draw based on only one point.
		{
			// Only a point was added
			
			if ( mNextPosition > mWidth )
			{
				ScrollMapLeft(mNextPosition - mWidth);
			}
			
			// Draw the next segment
			mBitmap->Lock();
			mView->SetPenSize(mLineWidth);
			mView->SetHighColor(mLineColor);
	
			BPoint p1,p2;
	
			p1.x = mCurrentPosition;
			p1.y = (float)mHeight - 1 - (float)mPoints[(theNewPosition-1+mDataWidth)%mDataWidth] * mVScale;
			p2.x = mNextPosition;
			p2.y = (float)mHeight - 1 - (float)mPoints[theNewPosition] * mVScale;
		
			mView->StrokeLine( p1,p2);
			mView->SetPenSize(1);
			mView->Sync();
			mBitmap->Unlock();
			// Now Update the Real View
	
			Window()->Lock();		
			DrawBitmap(mBitmap);
			Window()->Unlock();
			
			mCurrentPosition +=mHScale;
			mNextPosition += mHScale;
		}
	}
}
	
void
TGraphView::ScrollMapLeft(long inX)
{

	// Move the bitmap left
	char *theBits = (char*)mBitmap->Bits();
	short theMapWidth = mBitmap->BytesPerRow();
	char *theCurrentBit;
	
	mBitmap->Lock();

	for ( short i=0; i< mHeight; i++ )
	{
		for (theCurrentBit = theBits; theCurrentBit <= theBits - inX + mWidth; theCurrentBit++)
		{
			*theCurrentBit = *(theCurrentBit+inX);
		}
		
		theBits+= theMapWidth;
	}
	
	mView->SetHighColor(mViewColor);
	mView->FillRect( BRect(mWidth-inX+1,0,mWidth,mHeight) );
	// Draw the Grid Marks
	if ( mGridInterval )
	{
		long theGridNum = mMax / mGridInterval;
		BPoint p1,p2;
		p1.x = mWidth-inX+1;
		p1.y = mHeight;
		p2.x = mWidth;
		
		mView->SetPenSize(1);
		mView->BeginLineArray( theGridNum );
		
		for ( long j=0;j<=theGridNum;j++)
		{
			p1.y -=  mVScale * mGridInterval;
			p2.y = p1.y;
			
			mView->AddLine(p1,p2,mGridColor);
		}
		mView->EndLineArray();
		mView->SetPenSize(1);
	}

	mBitmap->Unlock();
	
	mCurrentPosition-=inX;
	mNextPosition -=inX;
	
	
}

void	
TGraphView::DrawEverything()
{	
	BPoint p1,p2;
	long thePoint,theNextPoint;

	mBitmap->Lock();
	mView->SetHighColor(mViewColor);
	mView->FillRect(mView->Frame());

	// Draw the Grid Marks
	if ( mGridInterval )
	{
		long theGridNum = mMax / mGridInterval;
	
		p1.x = 0;
		p1.y = mHeight;
		p2.x = mWidth;
		
		mView->SetPenSize(1);
		mView->BeginLineArray( theGridNum );
		
		for ( long j=0;j<=theGridNum;j++)
		{
			p1.y -=  mVScale * mGridInterval;
			p2.y = p1.y;
			
			mView->AddLine(p1,p2,mGridColor);
		}
		mView->EndLineArray();
	}
			

	
	mCurrentPosition = mWidth - mHScale * ( mNumOfPoints - 1);
	if (mCurrentPosition > 0)
		mCurrentPosition = 0;
	
	
	thePoint = mStart;
	theNextPoint = (thePoint + 1) % mDataWidth;	
	
	p1.x = mCurrentPosition;
	p1.y = (float)mHeight - 1 - (float)mPoints[thePoint] * mVScale;
	p2.x = mCurrentPosition+mHScale;
	p2.y = (float)mHeight - 1 - (float)mPoints[theNextPoint] * mVScale;
	
	mView->SetPenSize(mLineWidth);
	mView->SetHighColor(mLineColor);
	
	for ( long i=0; i < mNumOfPoints - 1; i++)
	{
		mView->StrokeLine(p1,p2);
		
		thePoint = theNextPoint;
		theNextPoint = (thePoint + 1) % mDataWidth;	
		mCurrentPosition += mHScale;
		
		p1=p2;
		p2.x = mCurrentPosition+mHScale;
		p2.y = (float)mHeight - 1 - (float)mPoints[theNextPoint] * mVScale; 
		
	}
	
	mView->Sync();
	mBitmap->Unlock();
		
	mNextPosition = mCurrentPosition+mHScale;
	//mCurrentPosition-=mHScale;


}

void
TGraphView::SetMinMax(long inMin)
{
	mMinMax = inMin;
	if (mMax < inMin )
	{
		mMax = mMinMax;	
		mVScale = (float) mHeight / mMax;
	}
	DrawEverything();
}

void
TGraphView::SetHScale(long s)
{
	if ( s != mHScale )
	{
		long theOldWidth = mDataWidth;
		
		mDataWidth = mWidth / mHScale + 1;
		
		
		// Allocate the Data Array
		long *theNewPoints = new long[mDataWidth];
		long theOldIndex = (mStart + mNumOfPoints) % theOldWidth;
		
		for (long i=mDataWidth-1;i>=0; i--)
		{
			// This may copy too many
			theNewPoints[i] = mPoints[theOldIndex];
			
			theOldIndex = (theOldIndex+theOldWidth-1) % theOldWidth;
		}
		
		delete[] mPoints;
		
		if (mDataWidth > theOldWidth)
		{
			mStart = mDataWidth - mNumOfPoints;
		}
		else
		{
			mStart = 0;
			mNumOfPoints = mDataWidth;
		}
		
		mPoints = theNewPoints;

		DrawEverything();
	}
}

void
TGraphView::SetGridInterval(long i)
{
	mGridInterval = i;
	DrawEverything();
}

void
TGraphView::ClearData()
{
	mNumOfPoints = 0;
	mStart = 0;
	
	DrawEverything();
}

void
TGraphView::SetLineWidth(long w)
{
	mLineWidth = w;
	
	DrawEverything();
}

	
	
	
	