// TGraphView.h
// By Trevor Walker

// This class will graph data points (integer) over time.  The graph is
// double buffered to increase performance.  The graph is scaled to the Maximum
// value currently in the graph.  A minimum scale factor is specifable (SetMinMax).
// The horizontal space between points is also settable (SetHScale).  The 
// graph colors are configurable.  

// A horizontal grid is also possible ( SetGridInterval ).

// To use, create a TGraphView, set the options, add it to it's parent view, then
// call AddPoint to add points of data.

// Note:  This version is not resizable.  To resize the graph it must by destroyed
// and recreated (which delete the data).

// Use of this code is permitted in all shareware as long as credit
// is given.  For use in commercial software please contact me at
// twalker@cc.edu.

#pragma once

class BBitmap;
//class BView;
#include <View.h>

class TGraphView : public BView
{
public:
	TGraphView(BRect frame, char *name, ulong mode);
	~TGraphView();
	
	virtual void 	Draw(BRect);
	
			void	AddPoint(long point);
			
			void	ScrollMapLeft(long);
			void	DrawEverything();
			
			void	SetMinMax(long inMin);
			void	SetLineColor(rgb_color c) { mLineColor = c; }
	virtual	void	SetViewColor(rgb_color c) { mViewColor = c; inherited::SetViewColor(c);}
			void	SetGridColor(rgb_color c) { mGridColor = c; }
			
			void	SetHScale(long s);
			void	SetGridInterval(long i);
			void	SetLineWidth(long w);
			
			void	ClearData();
			
private:

BBitmap*	mBitmap;
BView*		mView;

long		mWidth,mHeight;
long		mCurrentPosition,mNextPosition;

rgb_color	mLineColor,mViewColor,mGridColor;
long		mDataWidth;
float		mHScale,mVScale;

long		mStart,mNumOfPoints;

long		mMax,mMinMax;

long		*mPoints;
long 		mGridInterval;

long		mLineWidth;
};