// TLedView.cpp

/*
* $Log: TLedView.cpp,v $
* Revision 1.1  1997/04/05 15:58:52  elvis
* Initial revision
*
*/

#include "TLedView.h"
#include "colors.h"

TLedView::TLedView(BRect frame, char * name, ulong resize)
	: BView(frame,name,resize,B_WILL_DRAW)
{
	mMax = 100;
	mScale = 10;
	
	mTicks = mMax / mScale; 
	if ( mTicks * mScale < mMax ) mTicks++;
	
	mWarning1 = 50;
	mWarning2 = 80;
	
	mWarningMark1 = mWarning1 / mScale;
	mWarningMark2 = mWarning2 / mScale;
	
	mColor1 = green;
	mColor2 = yellow;
	mColor3 = red;
	
	//mMarkColor = mColor1;
	mMarkColor1.red = 60;
	mMarkColor1.green = 90;
	mMarkColor1.blue = 60;
	
	mMarkColor2.red = 100;
	mMarkColor2.green = 100;
	mMarkColor2.blue = 30;
	
	mMarkColor3.red = 100;
	mMarkColor3.green = 60;
	mMarkColor3.blue = 60;

	mCurrentValue = 0;
	
	frame.OffsetTo(0,0);
	frame.InsetBy(1,1);
	mBorderRect=frame;
	
	SetViewColor(black);
}

void
TLedView::Draw(BRect frame)
{
	BRect theRect = Bounds();
	
	SetHighColor(ViewColor());
	FillRect(theRect);
	
	long theCurrentMark = 0;
	
	theRect = mBorderRect;
	long thePosition = theRect.bottom;
	
	rgb_color theColor;
	
	BeginLineArray(mTicks * 2);
	
	for (long i=0; i<mTicks; i++,theCurrentMark+=mScale)
	{
		if ( mCurrentValue <= theCurrentMark)
		{
			if ( i < mWarningMark1 )
				theColor = mMarkColor1;
			else if ( i < mWarningMark2 )
				theColor = mMarkColor2;
			else
				theColor = mMarkColor3;
		}
		else
		{
			if ( i < mWarningMark1 )
				theColor = mColor1;
			else if ( i < mWarningMark2 )
				theColor = mColor2;
			else
				theColor = mColor3;
		}
			
		AddLine(BPoint(theRect.left,thePosition), BPoint(theRect.right,thePosition), theColor);
		thePosition--;
		AddLine(BPoint(theRect.left,thePosition), BPoint(theRect.right,thePosition), theColor);
		thePosition-=2;
		
		
	}
	
	EndLineArray();

}

void
TLedView::DrawMarks(long oldValue, long newValue)
{
	long theStart,theEnd;
	if (oldValue < newValue )
	{
		theStart = oldValue/mScale;
		theEnd = newValue/mScale;
		if (theEnd >= mTicks ) theEnd = mTicks -1;
	}
	else
	{
		theStart = newValue/mScale;
		theEnd = oldValue/mScale;
		if (theEnd >= mTicks ) theEnd = mTicks -1;
	}

	BRect theRect = mBorderRect;
	
	long theCurrentMark = theStart * mScale;
	long thePosition = theRect.bottom ;
	thePosition -= theStart * 3;
	
	rgb_color theColor;

	for (long i=theStart; i<=theEnd && i < mTicks; i++,theCurrentMark+=mScale)
	{
		if (newValue <= theCurrentMark)
		{
			if ( i < mWarningMark1 )
				theColor = mMarkColor1;
			else if ( i < mWarningMark2 )
				theColor = mMarkColor2;
			else
				theColor = mMarkColor3;
		}
		else
		{
			if ( i < mWarningMark1 )
				theColor = mColor1;
			else if ( i < mWarningMark2 )
				theColor = mColor2;
			else
				theColor = mColor3;
		}
		
		SetHighColor(theColor);
		StrokeLine(BPoint(theRect.left,thePosition), BPoint(theRect.right,thePosition));
		thePosition--;
		StrokeLine(BPoint(theRect.left,thePosition), BPoint(theRect.right,thePosition));
		thePosition-=2;
	}
}

void
TLedView::SetCurrentValue(long v)
{
	if ( v != mCurrentValue )
	{
		if ( v > mMax )
			v = mMax;
		else
		if ( v < 0 )
			v = 0;
			
		DrawMarks( mCurrentValue,v);
		mCurrentValue = v;
	}
}

void
TLedView::SetScale(long s)
{
	mScale = s;
	
	mWarningMark1 = mWarning1 / mScale;
	mWarningMark2 = mWarning2 / mScale;
	
	if ( Window() ) Draw(Bounds());

}
			
			

void
TLedView::SetMax(long m)
{
	mMax = m;
	mTicks = mMax / mScale; 
	if ( mTicks * mScale < mMax ) mTicks++;
	
	if ( Window() ) Draw(Bounds());
}

void
TLedView::SetWarningPoint(long p1, long p2)
{
	mWarning1 = p1;
	mWarning2 = p2;
	
	mWarningMark1 = mWarning1 / mScale;
	mWarningMark2 = mWarning2 / mScale;

	if ( Window() ) Draw(Bounds());

}

void
TLedView::SetBarColor1(rgb_color c)
{
	mColor1 = c;
	if ( Window() ) Invalidate() ;
}

void
TLedView::SetBarColor2(rgb_color c)
{
	mColor2 = c;
	if ( Window() ) Invalidate() ;
}

void
TLedView::SetBarColor3(rgb_color c)
{
	mColor3 = c;
	if ( Window() ) Invalidate() ;
}

void
TLedView::SetMarkColor1(rgb_color c)
{
	mMarkColor1 = c;
	if ( Window() ) Invalidate() ;
}

void
TLedView::SetMarkColor2(rgb_color c)
{
	mMarkColor2 = c;
	if ( Window() ) Invalidate() ;
}

void
TLedView::SetMarkColor3(rgb_color c)
{
	mMarkColor3 = c;
	if ( Window() ) Invalidate() ;
}
