// TSwissArmyView.cpp

/*
* $Log: TSwissArmyView.cpp,v $
* Revision 1.1  1997/04/05 15:58:52  elvis
* Initial revision
*
*/

#include "TSwissArmyView.h"
#include "TCaptionWindow.h"
#include "debug.h"


// Constructor and Destructor

TSwissArmyView::TSwissArmyView(BRect inRect, const char *inName, ulong inResize, ulong inFlags)
	: BView(inRect, inName, inResize, inFlags)
{
	mClicks = 1;		
	mTime = 0;			
	
	mDragEnabled = false;					
	mDragThreshold = 3; 
							
	mContextThreshold = 660000;					
	mContextEnabled = false;
							
	mHighlightEnabled = false;
	mHighlight = T_NO_HIGHLIGHT;		

	mContextMenu = NULL;
	
}

TSwissArmyView::~TSwissArmyView()
{
}

// BView Methods

void
TSwissArmyView::MouseDown(BPoint inPoint)
{
	ulong 	theButtons;
	long  	theDistance;
	long	theClicks;
	BPoint	theCurrentPoint;
	double  theClickSpeed;
	BRect	theBounds = Bounds();
	double 	theTime;
	
	bool	tookAction = false;
	long	theOriginalHighlight = mHighlight;
	
	// Make sure to clear any help that is shown
	if ( gCaptionWindow )
	{
		gCaptionWindow->ViewExited(this, ConvertToScreen(inPoint), true);
	}

	// Strip info from message
	theButtons = Window()->CurrentMessage()->FindLong("buttons");
	theClicks = Window()->CurrentMessage()->FindLong("clicks");
	theTime = Window()->CurrentMessage()->FindDouble("when");

	theCurrentPoint = inPoint;
	
	get_click_speed(&theClickSpeed);
	
	if ( theButtons == B_PRIMARY_MOUSE_BUTTON )
	{
		// Check if it is a double,triple,etc. click
		if ( theTime - mTime < theClickSpeed )
			mClicks++;
		else
			mClicks = 1;
			
		mTime = theTime;
			
		Highlight(T_MOUSEDOWN_HIGHLIGHT);
		tookAction = HandleDown(inPoint);
		
		while ( theButtons && ! tookAction )
		{
			if ( mDragEnabled )
			{
				theDistance = (inPoint.x - theCurrentPoint.x) * (inPoint.x - theCurrentPoint.x)
				 			+ (inPoint.y - theCurrentPoint.y) * (inPoint.y - theCurrentPoint.y);
				if ( theDistance > mDragThreshold * mDragThreshold || ! theBounds.Contains( theCurrentPoint ) )
				{
					// Start a drag if distance exceeds threshold or point leaves view
					StartDrag(inPoint);
					tookAction = true;
				}
			}
			else
			{
				if ( ! theBounds.Contains( theCurrentPoint ) )
				{
					// Not Dragging, so watch whether mouse is in view.  If it leave, 
					// unhighlight
					Highlight(T_NO_HIGHLIGHT);
				}
			}
			
			if ( mContextEnabled && mContextThreshold > 0 && system_time() - theTime > mContextThreshold )
			{
				// Show the context menu if enabled and time threshold exceeded.
				ContextMenu(theCurrentPoint);
				tookAction = true;
			}
			
			snooze(20.0 *1000);
			GetMouse(&theCurrentPoint,&theButtons);
		}
		if ( ! tookAction && theBounds.Contains(theCurrentPoint) )
		{
			HandleUp(theCurrentPoint);
		}
		Highlight(theOriginalHighlight);
	}
	else
	if (theButtons == B_SECONDARY_MOUSE_BUTTON )
	{
		// reset the double click variables
		mClicks = 1;
		mTime = 0;
		
		HandleDown(inPoint);
		if ( mContextEnabled )
		{
			ContextMenu(theCurrentPoint);
		}
	}
}

void
TSwissArmyView::MouseMoved(BPoint inPoint, ulong inState, BMessage *inMsg)
{
	if ( inState == B_ENTERED_VIEW )
	{
		if ( inMsg == NULL )
		{
			Highlight(T_MOUSEOVER_HIGHLIGHT);
			
			// Check for caption display
			if ( mCaptionEnabled )
			{			
				if ( gCaptionWindow == NULL )
					new TCaptionWindow();
				
				gCaptionWindow->ViewEntered(this,ConvertToScreen(Bounds()),mCaption);	
					
			}
		}	
		
	}
	else
	if ( inState == B_EXITED_VIEW   )
	{
		if (mHighlight != T_NO_HIGHLIGHT)
			Highlight(T_NO_HIGHLIGHT);
			
		if ( gCaptionWindow )
		{
			gCaptionWindow->ViewExited(this, ConvertToScreen(inPoint));
		}
	}
}

// Mouse Down Methods
bool
TSwissArmyView::HandleDown(BPoint inPoint)
{
	return false;  // Keep processing mouse down
}

void
TSwissArmyView::HandleUp(BPoint inPoint)
{
}

void
TSwissArmyView::StartDrag(BPoint inPoint)
{

}

void
TSwissArmyView::ContextMenu(BPoint inPoint)
{
	if ( mContextMenu != NULL )
	{
		BRect theBounds = Bounds();
		ConvertToScreen(&theBounds);
		ConvertToScreen(&inPoint);
		mContextMenu->Go(inPoint, true, false, theBounds);
	}
}

void
TSwissArmyView::Highlight(long inState)
{
	if (mHighlightEnabled)
	{
		mHighlight = inState;
		Draw(Bounds());
	}
}

void
TSwissArmyView::SetContextMenu(BPopUpMenu *inMenu)
{		
	mContextMenu = inMenu;
	
	if ( mContextMenu != NULL )
	{
		mContextEnabled = true;
	}
	else
		mContextEnabled = false;
}

void
TSwissArmyView::SetCaption(const char *inCaption)
{
	if ( *inCaption == '\0' )
	{
		mCaptionEnabled = false;
	}
	else
	{
		strncpy(mCaption,inCaption,50);
		if ( gCaptionWindow) gCaptionWindow->SetCaption(this,mCaption);
		mCaptionEnabled = true;
	}
	
}
