//TSwissArmyView.h

// Value-added class for views.  Just add it in and derive from it like
// you would from BView.

// TSwissArmyView is a value-added class that makes implementing certain
// user interface feature easier.  These include contexxt sensitive popup
// menus, drag initiation, and mouse position sensitivity.

// Dependencies:
// 	TCaptionWindow

// Features:
//	- Drag initiation based on threshold of mouse move or cursor leaving view
//	- Context Sensitive Menu initiated from either the secondary button or
//		the primary button (after delay).
//  - Mouse Position Sensitivity.
//	- Popup Caption Help

// To Use:
// Derive your view class from TSwissArmyView.  
//
// To Activate Feature:
// Context Menu - Create BPopUpMenu and Call SetContextMenu().
// 				  OR override ContextMenu() method to provide your own
//				  method of context sensitive help.
// Drag	& Drop  - Override StartDrag method.  Make sure to set mDragEnabled
//				  to true.
// Mouse Clicks - Override HandleDown and HandleUp.  HandleDown is called
//				  when the mouse is first clicked.  HandleUp is called
//				  a mouse button release (but only if a Drag or Menu was
//				  initiated).
// Position Sensitivity - Write your draw method to respect mHighlight
//				  variable.  mHighlight will be changed appropriately automatically.
// Popup Caption Help - Set the Views caption with SetCaption function.  
//				  Caption will appear when appropriate.  Warning: Buggy.

// For further information, set comments below.

// Use of this code is permitted in all shareware as long as credit
// is given.  For use in commercial software please contact me at
// twalker@cc.edu.
#pragma once

enum {
	T_NO_HIGHLIGHT = 0,
	T_MOUSEOVER_HIGHLIGHT = 1,
	T_MOUSEDOWN_HIGHLIGHT = 2
};

class TSwissArmyView : public BView
{
public:
	TSwissArmyView(BRect inRect, const char *inName, ulong inResize, ulong inFlags);
	virtual ~TSwissArmyView();
	
	virtual void 	MouseDown(BPoint inPoint);
	virtual void	MouseMoved(BPoint inPoint, ulong inState, BMessage *inMsg);


	// Mouse Down Methods
	virtual bool	HandleDown(BPoint inPoint);
						// Handle the initial mouse click.  Return true
						// if you have completely handled the click,
						// MouseDown will wait for button to be release
						// and take action based on that.
						// Default method is empty.
	virtual	void	HandleUp(BPoint inPoint);
						// Handle the release of the mouse button.  
						// If this is called, mouse is still in view
						// and no drag/context actions were taken.
						// Default method is empty.
	virtual void	StartDrag(BPoint inPoint);
						// Handle starting a drag.  inPoint is the
						// location of the original mouse click.
						// MouseDown handling is terminated after
						// StartDrag without calling HandleUp.
						// Default method is empty.
	virtual void	ContextMenu(BPoint inPoint);
						// Show context sensitive menu.  
						// MouseDown handling is terminated after
						// ContextMenu without calling HandleUp.
						// Default method shows mContextMenu if set.
	virtual void	Highlight(long inState);
						// Changes the current look of the view.
						// This function should set mHighlight and redraw
						// the view approriately.
						// 0 = T_NO_HIGHLIGHT -> Unhighlighted;
						// 1 = T_MOUSEOVER_HIGHLIGHT -> this is the highlight
						// when the mouse passes over;
						// 2 = T_MOUSEDOWN_HIGHLIGHT -> this is the highlight
						// when the mouse button is pressed and the mouse
						// is in the view;		
						// Highlights doesn't necessary deal with whether something
						// is selected or not.  Unselected things can be highlighted
						// and selected things can be highlighted.		
	// Setup Methods
			void	SetContextMenu(BPopUpMenu *inMenu);
						// Sets the context sensitive menu that is display
						// by the default ContextMenu method.  Sets
						// mContextEnabled to true.
						// The menu becomes property of the view when
						// It is set.  It will be freed automatically.
			void	SetCaption(const char *inCaption);
						// Sets the caption that will display for help.
						// Sets mCaptionEnabled to true.
			void	SetCaptionEnabled(bool c) { mCaptionEnabled = c; }
			void	SetDragEnabled(bool d) { mDragEnabled = d; }
			void	SetDragThreshold(long t) { mDragThreshold = t; }
			void	SetContextEnabled(bool c) { mContextEnabled = c; }
			void	SetContextThreshold(double t) { mContextThreshold = t; }
			void	SetHighlightEnabled(bool h) { mHighlightEnabled = h; }
			
			bool	CaptionEnabled() { return mCaptionEnabled; }
			bool	DragEnabled() { return mDragEnabled; }
			long	DragThreshold() { return mDragThreshold; }
			bool	ContextEnabled() { return mContextEnabled; }
			double	ContextThreshold() { return mContextThreshold; }
			bool	HighlightEnabled() { return mHighlightEnabled; }
protected:
	long	mClicks;		// Number of Clicks for current mouse event
	double	mTime;			// Time of last mouse click.
	
	bool	mDragEnabled;	// If dragging is enabled.  StartDrag 
							// will not be called unless this is true.
							// false by default.
	long	mDragThreshold; // Distance (in Pixels) mouse can move before
							// StartDrag is called. Default is 3 pixels.
	double	mContextThreshold;	// Time before context sensitive menu is
							// shown on Primary click.  Set to -1 for no
							// context menu on primary click.
							// Default is one second
	bool	mContextEnabled;// If context menu is enabled. ContextMenu
							// will not be shown unless this is true;
							// false by default.
	bool	mHighlightEnabled; // If automatic mouseover and mousedown
							// highlighting is enabled.  
							// false by default.
	long	mHighlight;		// Current Highlight state.  See Highlight above.
	bool	mCaptionEnabled; // Whether to show caption automatically.
							// false by default.
	char	mCaption[50];	// Caption that is displayed.  Use SetCaption to set.
	
	
	BPopUpMenu	*mContextMenu;	// Pointer to the contxt menu that is shown by
							// default method. None shown if BMenu is null.
	
};